#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2021-12-19
#
# WATO file for the cve_2021_44228 plugins (bakery and check)
#
# 2021-12.19: added WATO options scan_logback, log4j_1, no_symlink, scan_zip, silent
#

from cmk.gui.i18n import _
from cmk.gui.valuespec import (
    Dictionary,
    Integer,
    Tuple,
    Float,
)

from cmk.gui.plugins.wato import (
    rulespec_registry,
    RulespecGroupCheckParametersOperatingSystem,
    CheckParameterRulespecWithItem,
)

bakery_plugin_version = '2021-12-21-0.0.1b'

##############################################################
#
# Levels for return values check plugin cve_2021_44228_log4j
#
##############################################################
_items_on_info = [
    ('files_vulnerable', 'Files vulnerable'),
    ('files_potential_vulnerable', 'Files potential vulnerable'),
    ('files_mitigated', 'Files mitigated'),
    ('files_scanned', 'Files scanned'),
    ('directories_scanned', 'Directories scanned'),
    ('run_time', 'Run time'),
    ('last_run', 'Last run'),
    ('scanner_version', 'logresso scanner version'),
    ('scan_options', 'Scan options'),
    ('script_version', 'Script version'),
]


def _valuespec_cve_2021_44228_log4j():
    return Dictionary(
        elements=[
            ('files_vulnerable',
             Tuple(
                 title=_('Files vulnerable'),
                 elements=[
                     Integer(title=_('Warning at'), minvalue=0, unit=_('Files'), default_value=1),
                     Integer(title=_('Critical at'), minvalue=0, unit=_('Files'), default_value=1),
                 ])),
            ('files_potential_vulnerable',
             Tuple(
                 title=_('Files potentially vulnerable'),
                 elements=[
                     Integer(title=_('Warning at'), minvalue=0, unit=_('Files'), default_value=1),
                     Integer(title=_('Critical at'), minvalue=0, unit=_('Files'), default_value=1),
                 ])),
            ('files_mitigated',
             Tuple(
                 title=_('Files mitigated'),
                 elements=[
                     Integer(title=_('Warning at'), minvalue=0, unit=_('Files'), default_value=1),
                     Integer(title=_('Critical at'), minvalue=0, unit=_('Files'), ),
                 ])),
            ('files_scanned',
             Tuple(
                 title=_('Files scanned'),
                 elements=[
                     Integer(title=_('Warning at'), minvalue=0, unit=_('Files'), ),
                     Integer(title=_('Critical at'), minvalue=0, unit=_('Files'), ),
                 ])),
            ('directories_scanned',
             Tuple(
                 title=_('Directories scanned'),
                 elements=[
                     Integer(title=_('Warning at'), minvalue=0, unit=_('Directories'), ),
                     Integer(title=_('Critical at'), minvalue=0, unit=_('Directories'), ),
                 ])),
            ('run_time',
             Tuple(
                 title=_('Run time'),
                 elements=[
                     Float(title=_('Warning at'), minvalue=0, unit=_('s'), ),
                     Float(title=_('Critical at'), minvalue=0, unit=_('s'), ),
                 ])),
            ('items_on_info',
             ListChoice(
                 title=_('Items to show up in the check info'),
                 help=_('Selected items will show up in the service info. '
                        'Default is "Files vulnerable" and "Files potential vulnerable"'),
                 choices=_items_on_info,
                 default_value=['files_vulnerable', 'files_potential_vulnerable'],
             )),
        ])


rulespec_registry.register(
    CheckParameterRulespecWithItem(
        check_group_name='cve_2021_44228_log4j',
        group=RulespecGroupCheckParametersOperatingSystem,
        parameter_valuespec=_valuespec_cve_2021_44228_log4j,
        title=lambda: _('CVE-2021-44228_log4j'),
        match_type='dict',
    ))

##############################################################
#
# Config for agent plugin cve_2021_44228_log4j.(sh|ps1)
#
##############################################################


from cmk.gui.cee.plugins.wato.agent_bakery.rulespecs.utils import (
    RulespecGroupMonitoringAgentsAgentPlugins,
)
from cmk.gui.plugins.wato import HostRulespec
from cmk.gui.valuespec import (
    CascadingDropdown,
    FixedValue,
    TextInput,
    TextUnicode,
    ListOfStrings,
    ListChoice,
    DropdownChoice,
)

_base_options_config_fix_files = (
    'fix_files',
    Dictionary(
        title=_('Fix files and backup'),
        elements=[
            ('force_fix',
             FixedValue(
                 '--force-fix',
                 title=_('Fix files. (Use at your own risk!)'),
                 totext=_('Files will be fixed'),
                 help=_('Do not prompt confirmation. Don\'t use this option unless you know what you are doing.')
             )),
            ('backup_dir',
             TextUnicode(
                 title=_('Backup directory'),
                 help=_(
                     'Specify backup file path. Remember the directory must exist '
                     'and scanner must be able to write there!'
                 ),
                 allow_empty=False,
             )),
            ('not_exclude_backup',
             FixedValue(
                 True,
                 title=_('Don\'t Exclude backup path'),
                 totext=_('Don\'t Exclude backup path from scanning'),
                 help=_('Backup path will be not excluded from scanning.'),
             )),
        ],
        required_keys=['backup_dir'],
    ),

)

_base_options_config_interval = (
    'interval',
    Integer(
        title=_('Cache time (min 600s)'),
        minvalue=600,
        unit=_('s'),
        default_value=86400,
        help=_('This is the caching time for the scanner output. Default is 86400s (one day). Minimum is 600s (10min)'),
    ),

)

_base_options_config_timeout = (
    'timeout',
    Integer(
        title=_('Scanner timeout (min 60s)'),
        minvalue=60,
        unit=_('s'),
        default_value=300,
        help=_('This is the maximum run time for the scanner. Default is 300s (5min). Minimum is 60s (1min)'),
    ),
)

_base_options_config_scan_logback = (
    'scan_logback',
    FixedValue(
        '--scan-logback',
        title=_('Scan for logback (CVE-2021-42550)'),
        totext=_('Scan for logback (CVE-2021-42550) enabled'),
        help=_('Enables scanning for logback CVE-2021-42550.'),
    )
)

_base_options_config_log4j_1 = (
    'log4j_1',
    FixedValue(
        '--scan-log4j1',
        title=_('Scan for log4j 1 versions (CVE-2021-4104)'),
        totext=_('Scan for log4j 1 versions (CVE-2021-4104) enabled'),
        help=_('Enables scanning for log4j 1 versions (CVE-2021-4104).'),
    )
)

_base_options_config_scan_zip = (
    'scan_zip',
    FixedValue(
        '--scan-zip',
        title=_('Scan zip files (increase timeout)'),
        totext=_('Scanning .zip files enabled'),
        help=_('Scan also .zip extension files. This option may slow down scanning.'),
    )
)

_base_options_config_no_symlink = (
    'no_symlink',
    FixedValue(
        '--no-symlink',
        title=_('Ignore symlinks'),
        totext=_('Ignore symlinks enabled'),
        help=_('Do not detect symlink as vulnerable file.'),
    )
)

_base_options_config_silent = (
    'silent',
    FixedValue(
        '--silent',
        title=_('Silent output'),
        totext=_('Silent output enabled'),
        help=_('Do not print anything until scan is completed. This will '
               'remove some progress messages from the scanner output'),
    )
)

_base_option_config_exclude_fs = (
    'exclude_fs',
    ListOfStrings(
        title=_('Exclude filesystems by type'),
        orientation='horizontal',
        allow_empty=False,
        valuespec=TextInput(allow_empty=False, regex='[a-zA-Z0-9\.]'),
        help=_('Exclude paths by file system type. nfs, nfs3, nfs4, cifs, '
               'tmpfs, devtmpfs, fuse.sshfs and iso9660 is ignored by default.'),
    )
)

_base_option_config_exclude_paths = (
    'exclude_paths',
    ListOfStrings(
        title=_('Exclude paths'),
        orientation='horizontal',
        allow_empty=False,
        valuespec=TextInput(allow_empty=False, regex='[^|<>]'),
        help=_('Exclude specified paths from the scanning'),
    )
)

_base_option_config_syslog = (
    'syslog',
    Dictionary(
        title=_('Enable syslog reporting'),
        elements=[
            ('syslog_server',
             TextUnicode(
                 title=_('Syslog server'),
                 help=_('IP-Address or hostname of the syslog server to log to.'),
                 allow_empty=False,
             )),
            ('syslog_port',
             Integer(
                 title=_('Syslog server Port'),
                 help=_('Port of the syslog server. Default ist 512.'),
                 default_value=514,
                 minvalue=1,
                 maxvalue=65535
             )),
            ('syslog_level',
             DropdownChoice(
                 title=_('Loglevel'),
                 help=_(
                     'Use "alert" level for SIEM integration. It will report vulnerable/potential vulnerable files.\n'
                     'Use "info" level for BI reporting. It reports also MITIGATED files. This is the default mode.\n'
                     'Use "debug" level for error reporting'),
                 choices=[
                     ('alert', _('Alert')),
                     ('info', _('Info')),
                     ('debug', _('Debug')),
                 ],
                 default_value='info',
             )),
        ],
        required_keys=['syslog_server']
    ),
)

_base_option_config_report = (
    'reporting',
    Dictionary(
        title=_('Enable file reporting'),
        elements=[
            ('report_dir',
             TextUnicode(
                 title=_('Report output directory'),
                 help=_('Specify report output directory. Remember the scanner must be able to write there!'),
                 allow_empty=False,
             )),
            ('report_format',
             DropdownChoice(
                 title=_('Report format'),
                 help=_(
                     'Generate log4j2_scan_report_yyyyMMdd_HHmmss.csv or '
                     'log4j2_scan_report_yyyyMMdd_HHmmss.json in the report directory.'),
                 choices=[
                     ('--report-csv', _('CSV')),
                     ('--report-json', _('JSON')),
                 ],
                 default_value='--report-csv',
             )),
            ('no_empty_report',
             FixedValue(
                 '--no-empty-report',
                 title=_('Don\'t create empty reports'),
                 totext=_('Don\'t create empty reports'),
                 help=_('Do not generate empty report.'),
             )),
        ],
        required_keys=['report_dir']
    ),
)

_base_options_config_debug = (
    'debug',
    FixedValue(
        '--debug',
        title=_('Debug scanner'),
        totext=_('Debug scanner enabled'),
        help=_('Print exception stacktrace for debugging.'),
    )
)

_base_options_config_trace = (
    'trace',
    FixedValue(
        '--trace',
        title=_('Trace scanner (Use only for troubleshooting!! It produces a lot ou output'),
        totext=_('Trace scanner enabled'),
        help=_('Print all directories and files while scanning.'),
    )
)


def _valuespec_agent_config_cve_2021_44228_log4j():
    return CascadingDropdown(
        title=_('CVE-2021-44228-log4j'),
        help=_(
            f'If you activate this option, then the agent plugin <tt>cve_2021_44228_log4j</tt> will be deployed. '
            f'This will scan for files with the CVE-2021-44228-log4j issue. (Plugin version: {bakery_plugin_version})'
        ),
        choices=[
            (
                'linux',
                _('Deploy Linux CVE-2021-44228-log4j agent plugin'),
                Dictionary(elements=[
                    ('search_path',
                     ListOfStrings(
                         title=_('Search paths'),
                         orientation='horizontal',
                         allow_empty=False,
                         valuespec=TextInput(allow_empty=False, regex='[^|<>]'),
                         help=_('Paths where the scanner searches for vulnerable files'),
                     )),
                    _base_options_config_scan_logback,
                    _base_options_config_log4j_1,
                    _base_options_config_scan_zip,
                    _base_options_config_fix_files,
                    _base_options_config_no_symlink,
                    _base_option_config_exclude_paths,
                    _base_option_config_exclude_fs,
                    _base_options_config_silent,
                    _base_options_config_interval,
                    _base_options_config_timeout,
                    _base_option_config_syslog,
                    _base_option_config_report,
                    _base_options_config_debug,
                    # _base_options_config_trace,  # run takes to long, produces to much output
                ]),
            ),
            (
                'windows',
                _('Deploy Windows CVE-2021-44228-log4j agent plugin'),
                Dictionary(elements=[
                    ('search_path',
                     ListOfStrings(
                         title=_('Search paths'),
                         orientation='horizontal',
                         allow_empty=False,
                         valuespec=TextInput(allow_empty=False, regex='[^|<>]'),
                         help=_('Paths where the scanner searches for vulnerable files'),
                     )),
                     ('drives_to_scan',
                     ListOfStrings(
                         title=_('Drives to scan'),
                         orientation='horizontal',
                         allow_empty=False,
                         valuespec=TextInput(size=1, maxlen=1, minlen=1, allow_empty=False, regex='[a-zA-Z]'),
                         help=_('This drives will be scanned, default is "--all-drives"'),
                         default_value=['C'],
                     )),
                    _base_options_config_scan_logback,
                    _base_options_config_log4j_1,
                    # _base_options_config_no_symlink,  # sym links on windows?
                    _base_options_config_scan_zip,
                    _base_options_config_fix_files,
                    _base_option_config_exclude_paths,
                    # _base_option_config_exclude_fs, # filesystem type on windows?
                    _base_options_config_silent,
                    _base_options_config_interval,
                    _base_options_config_timeout,
                    _base_option_config_syslog,
                    _base_option_config_report,
                    _base_options_config_debug,
                    # _base_options_config_trace,  # run takes to long, produces to much output
                ]),
            ),
            (None, _('Do not deploy the CVE-2021-44228-log4j agent plugin')),
        ],

    )


rulespec_registry.register(
    HostRulespec(
        group=RulespecGroupMonitoringAgentsAgentPlugins,
        name='agent_config:cve_2021_44228_log4j',
        valuespec=_valuespec_agent_config_cve_2021_44228_log4j,
    )
)
